local major = 1
local minor = 6
local lib = LibStub:NewLibrary("LibGoingPrice-"..major, minor)

if not lib then
	return	-- already loaded and no upgrade necessary
end

-- LibGoingPrice currently depends on LibExtraTip
--
local lib_tt = LibStub("LibExtraTip-1")

lib_tt:SetEmbedMode(false)
lib_tt:RegisterTooltip(GameTooltip)
lib_tt:RegisterTooltip(ItemRefTooltip)

lib.nameColor = "ffff5400"
lib.evilColor = "ffff1111"
lib.saintColor = "ff8888ff"
	
-- Defining LibGoingPrice also defines the GoingPrice main panel. But first
-- make sure the deprecated base addon "GoingPrice" didn't do it already.
--
if ( IsAddOnLoaded("GoingPrice") ) then
	DEFAULT_CHAT_FRAME:AddMessage("|c"..lib.evilColor.."Note:|r |c"..lib.nameColor.."GoingPrice|r base addon is no longer required. All base functionality has been factored into the |c"..lib.saintColor.."LibGoingPrice|r library. Please disable/remove the |c"..lib.nameColor.."GoingPrice|r base addon and from now on load only the |c"..lib.nameColor.."GoingPrice_<source>|r data source addons.")
else
	local panel = CreateFrame("Frame","GoingPrice_Panel")
	
	panel.parent = nil
	panel.name = "GoingPrice"
	
	local toc_keys = {"Version", "Author", "X-Category", "X-Email", "X-Website"}
	--local toc_vals = {}
	local toc_vals = {["Version"] = string.format("%2.1f",major) , ["Author"] = "|cffffffffNeikos", ["X-Category"] = "|cffffffffAuction, Economy, Tooltip", ["X-Email"] = "neikos@goingpriceaddon.com", ["X-Website"] = "http://goingpriceaddon.com"}
	
	local ft = panel:CreateFontString()
	
	ft:SetPoint('TOPLEFT', panel, 'TOPLEFT', 20, -20)
	ft:SetFontObject(GameFontNormalLarge)
	ft:SetText(panel.name)
	
	--local notes = GetAddOnMetadata(arg,"Notes")
	local notes = "Get a good idea of the auction house value of any item you mouseover." 
	
	local fn = panel:CreateFontString()
	
	fn:SetPoint('TOPLEFT', panel, 'TOPLEFT', 20, -50)
	fn:SetWidth(350)
	fn:SetFontObject(GameFontNormalSmall)
	fn:SetJustifyH("LEFT")
	fn:SetText("|cffffffff"..notes.." Expand to configure data source(s).")
	
	for i,k in ipairs(toc_keys) do
	
	    --local v = GetAddOnMetadata(arg,k)
	    --toc_vals[k] = v
	
	    local l = panel:CreateFontString()
	    local r = panel:CreateFontString()
	
	    l:SetPoint('TOPRIGHT', panel, 'TOPLEFT', 100, -90-20*(i-1))
	    r:SetPoint('TOPLEFT', l, 'TOPRIGHT', 10, 0)
	
	    l:SetFontObject(GameFontNormalSmall)
	    r:SetFontObject(GameFontNormalSmall)
	
	    k_deXed = string.gsub(k,"^X.","")
	
	    l:SetText(k_deXed)
	    --r:SetText("|cff7777ff"..v)
	    r:SetText("|cff7777ff"..toc_vals[k])
	end
	
	InterfaceOptions_AddCategory(panel)
	
	DEFAULT_CHAT_FRAME:AddMessage("|cffff5400GoingPrice Library version "
	                                ..toc_vals.Version .." Loaded.")
end

local function GetItemId(itemName)
	local item,itemLink = GetItemInfo(itemName)
	return tonumber(string.match(itemLink,"|Hitem:(%d+):.*|h"))
end

function lib:DisplaySourceTitle(tooltip,id)

	local pd = _G[self._name.."_PullDateCb"]
	local sv = _G[self._name.."_SV"]
	local data = _G[self._name.."_Data"]

	if pd:GetChecked() and sv._pullDate.keep then
		value = data[id][#data[id]]
		lib_tt:AddDoubleLine(tooltip,
			self._source,"("..date("%b %d, %Y %I:%M %p",value)..")")
	else
		lib_tt:AddLine(tooltip,self._source)
	end
end

function lib:GetCallback()
	self._callback = self._callback or function(tooltip,item)
	    local id = GetItemId(item)
		local wasSourceTitleDisplayed = false
		local data = _G[self._name.."_Data"]
		local sv = _G[self._name.."_SV"]

	
		if data[id] then
	
			local key = sv._firstKey
	
			while key do
				if sv[key].keep then
	
					local state = self[key]
					local value = data[id][sv._index[key]]
	
					if value then
	
						if not ( wasSourceTitleDisplayed or key == "Per stack" ) then
							self:DisplaySourceTitle(tooltip,id)
							wasSourceTitleDisplayed = true
						end
	
						if state.type == "integer" then
							lib_tt:AddDoubleLine(tooltip,
									"   "..key,value,1,1,1,1,1,1)
						elseif state.type == "date" then
							lib_tt:AddDoubleLine(tooltip,
									"   "..key,date("%b %d, %Y %I:%M %p",
									value),1,1,1,1,1,1)
						elseif state.type == "price" then
							if key == "Per stack" then
								local _r1,_r2,_r3,_r4,_r5,_r6,_r7,maxStack,_r9,_r0
									= GetItemInfo(id)
								if maxStack > 1 then
	
									if not wasSourceTitleDisplayed then
										self:DisplaySourceTitle(tooltip,id)
										wasSourceTitleDisplayed = true
									end
	
									lib_tt:AddMoneyLine(tooltip,
										"   "..key.." ("..maxStack..")",
										value,1,1,1)
								end
							else	
								lib_tt:AddMoneyLine(tooltip,"   "..key,value,1,1,1)
							end
						end
					end
				end
	
				key = sv[key].next
			end
		end
	end
	return self._callback
end

function lib:CopyState(s)
	local d = {}

	d._firstKey = s._firstKey
	d._lastKey = s._lastKey

	for i,k in ipairs(self._keys)  do
		d[k] = {
				checked = s[k].checked,
				keep = s[k].keep,
				prev = s[k].prev,
				next = s[k].next
			}
	end

	d._index = {}

	for k,i in pairs(s._index) do
		d._index[k] = i
	end

	d._pullDate = 	{
						checked = s._pullDate.checked,
						keep = s._pullDate.keep,
					}

	d._discard = 	{
						checked = s._discard.checked,
					}

	if s._reKeep then
		d._reKeep = {}
		for k,v in pairs(s._reKeep) do
			d._reKeep[k] = v
		end
	end

	d._tooltip_order = s._tooltip_order

	d._lib_major_version = s._lib_major_version or major
	d._lib_minor_version = s._lib_minor_version or minor

	return d
end

function lib:UpdatePreview(state)

	local panel = _G[self._name.."_Panel"] -- Get the addon's sub-panel object.

    for i,v in pairs(self._keys) do
		_G[self._name.."_PreviewItem_"..i]:Hide()
	end

	local i = 1

	local k = state._firstKey

	while k do
		local button = _G[self._name.."_PreviewItem_"..self._index[k]]

		button:SetPoint("TOPLEFT",
						panel,
						"TOPLEFT",
						200 ,
						-25 - (25 * i))

	    local sv = _G[self._name.."_SV"]

		if sv[k].keep then
			_G[button:GetName().."Text"]:SetTextColor(1,1,1)
		else
			_G[button:GetName().."Text"]:SetTextColor(0.5,0.5,0.5)
		end

		button:Show()

		k = state[k].next
		i = i + 1
	end
end

local function updateSliderText(slider)
	local sliderText = _G[slider:GetName().."Text"]
	sliderText:SetText(slider._basetext.."("..slider:GetValue()..")|r")
end

function lib:LoadPanelUI(state)

	-- Statistic checkbox's
	for i,k in ipairs(self._keys) do
		local cb = getglobal(self._name.."_StatCb_"..i)

		-- Gray out discarded Stats
		--
		if state[k].keep then
           	getglobal(cb:GetName() .. "Text"):SetTextColor(1,1,1)
		else
           	getglobal(cb:GetName() .. "Text"):SetTextColor(0.5,0.5,0.5)
		end
		cb:SetChecked(state[k].checked)
	end
	-- Pull date checkbox
	local pd = getglobal(self._name.."_PullDateCb")
	if state._pullDate.keep then
       	getglobal(pd:GetName() .. "Text"):SetTextColor(1,1,1)
	else
       	getglobal(pd:GetName() .. "Text"):SetTextColor(0.5,0.5,0.5)
	end
	pd:SetChecked(state._pullDate.checked)

	-- Discard checkbox
	_G[self._name.."_DiscardCb"]:SetChecked(state._discard.checked)

	-- Slider
	local slider = _G[self._name.."_Slider"]
	slider:SetValue(state._tooltip_order)
	updateSliderText(slider)

	-- List box of checked items
	self:UpdatePreview(state)
end

function lib:ReIndex()

	-- Discard unwanted statistics

	local newi = 0

	local newIndex = {}
	local newiXoldi = {}
	local sv = _G[self._name.."_SV"]

	for i,k in ipairs(self._keys) do
		if sv[k].keep then
			newi = newi + 1
			newIndex[k] = newi
			newiXoldi[newi] = sv._index[k]
		end
	end

	return newi,newiXoldi,newIndex
end

-- m 	-> newSize
-- n 	-> oldSize
-- MxN 	-> mapping from new index array to old index array
-- newIndex 	-> new mapping from keys to array index of a row of data

function lib:ShrinkData(maxNewi,newiXoldi,newIndex)

	local sv = _G[self._name.."_SV"]
	local data = _G[self._name.."_Data"]

	local v_new_str = "{"

	for i=1,maxNewi do
		v_new_str = v_new_str..tostring(i)..","
	end

	if sv._pullDate.keep then
		v_new_str = v_new_str..tostring(maxNewi+1)..","
	end

	v_new_str = v_new_str.."}"

	local new_data_ids = {}

	local empty_count = 0

	for id,v in pairs(data) do

		local f = loadstring("return "..v_new_str)

		local v_new = f()

		local empty = true

		for i=1,maxNewi do
			v_new[i] = v[newiXoldi[i]]
			if v_new[i] then empty = false end
		end

		if not empty then
			new_data_ids[#new_data_ids + 1] = id
		else
			empty_count = empty_count + 1
		end

		-- Discard the pull date if unchecked

		if sv._pullDate.keep then
			v_new[maxNewi+1] = v[#v]
		end

		v = nil
		data[id] = v_new
	end

	if empty_count > 0 then

		local new_data_str = "{"

		for i=1,#new_data_ids do
			new_data_str = new_data_str.."["..tostring(new_data_ids[i]).."] = 1,"
		end

		new_data_ids = nil -- Done with new_data_ids for empty_count > 0 case.
		--collectgarbage("collect")

		new_data_str = new_data_str.."}"

		local g = loadstring("return "..new_data_str)
		local new_data = g()

		new_data_str = nil -- Done with new_data_str
		--collectgarbage("collect")

		for id,_ in pairs(data) do
			if new_data[id] then
				new_data[id] = data[id]
			end
			data[id] = nil
		end

		data = nil -- Done with data
		_G[self._name.."_Data"] = new_data -- re-assign the global
		new_data = nil -- Done with new_data
		--collectgarbage("collect")
	else
		new_data_ids = nil -- Done with new_data_ids for empty_count <= 0 case.
	end

	--collectgarbage("collect")

	sv._index = nil
	collectgarbage("collect")

	sv._index = newIndex
end

local function okay(panel)

	local addon = _G[panel.name]
	local sv = _G[addon._name.."_SV"]

    --DEFAULT_CHAT_FRAME:AddMessage("|c"..lib.nameColor.."sv._tooltip_order = "..sv._tooltip_order or "nil")
    --DEFAULT_CHAT_FRAME:AddMessage("|c"..lib.nameColor.."addon._temp._tooltip_order = "..addon._temp._tooltip_order or "nil")
	if sv._tooltip_order ~= addon._temp._tooltip_order then 
    --DEFAULT_CHAT_FRAME:AddMessage("|c"..lib.saintColor.."About to change the tooltip order")
		addon:ChangeTooltipOrder(addon._temp._tooltip_order)
	end

	sv = addon:CopyState(addon._temp)
	_G[addon._name.."_SV"] = sv -- re-assign to global

	-- if "discardCheckbox" is checked and there are "non-gray (keep)" checked box that
	-- are unchecked, then shrink the data.

	local discard = _G[addon._name.."_DiscardCb"]
	local pullDate = _G[addon._name.."_PullDateCb"]

	if discard:GetChecked() then

		-- Do the work, but override saved checked state to false for now.
		discard:SetChecked(false)
		sv._discard.checked = false

		-- Check to see if any Stats are to be discarded
		for i,key in ipairs(addon._keys) do
			local cb = getglobal(addon._name.."_StatCb_"..i)
			if not cb:GetChecked() then
				sv[key].keep = false
				sv._reKeep[key] = nil 
				sv._index[key] = nil
           		getglobal(cb:GetName() .. "Text"):SetTextColor(0.5,0.5,0.5)
			end
		end

		-- Check to see if the pull date is to be discarded

		if not pullDate:GetChecked() then
			sv._pullDate.keep = false
			sv._reKeep._pullDate = nil
           	_G[pullDate:GetName()..Text]:SetTextColor(0.5,0.5,0.5)
		end

		local newSize,newToOldMap,newIndex = addon:ReIndex()
		addon:ShrinkData(newSize,newToOldMap,newIndex)
	end

	-- Willy nilly, check to see if stats are to be brought back

	for i,key in ipairs(addon._keys) do
		local cb = getglobal(addon._name.."_StatCb_"..i)
		if cb:GetChecked() then sv._reKeep[key] = true end
	end

	-- Willy nilly, check to see if pull date is to be brought back

	if pullDate:GetChecked() then
		sv._reKeep._pullDate = true
	end

	addon._temp = nil
end

local function cancel(panel)
	local addon = _G[panel.name]
	addon._temp = nil

	-- Rollback panel using addon's saved variable.
	addon:LoadPanelUI(_G[addon._name.."_SV"])
end

local function default(panel)

	local addon = _G[panel.name]

	addon._temp = addon:CopyState(addon)

	-- temp won't have 'discard' info so we have to set it here
	--
	local sv = _G[addon._name.."_SV"]

	for i,k in ipairs(addon._keys) do
		addon._temp[k].keep = sv[k].keep
	end

	addon._temp._pullDate.keep = sv._pullDate.keep

	addon._temp._index = {}
	for k,i in pairs(sv._index) do
		addon._temp._index[k] = i
	end

	addon:LoadPanelUI(addon._temp)
end

function lib:CreateOrGetTemp()
	-- if this is the first click in a "panel session", then temp is nil.
	if not self._temp then
		self._temp = self:CopyState(_G[self._name.."_SV"])
	end

	return self._temp
end

local function removeLinkedListKey(llist,k)

	local k_prev = llist[k].prev 
	local k_next = llist[k].next 

	if k_prev then
		llist[k_prev].next
			= llist[k].next
	else
		llist._firstKey
			= llist[k].next
	end

	if k_next then
		llist[k_next].prev
			= llist[k].prev
	else
		llist._lastKey
			= llist[k].prev
	end

	llist[k].prev = nil
	llist[k].next = nil
end

local function onStatisticClick(cb)

	local addon = _G[cb._addon_name]

	local temp = addon:CreateOrGetTemp()

	local k = cb._key

	local first = temp._firstKey
	local last = temp._lastKey

	if cb:GetChecked() then -- Add to the end of the list

		temp[k].checked = true

		if last then
			temp[last].next = k

			temp[k].prev = last

			temp[k].next = nil
			temp._lastKey = k

		else
			temp._firstKey = k
			temp._lastKey = k
		end

	else -- remove from the list

		temp[k].checked = false

		removeLinkedListKey(temp,k)

	end		

	addon:UpdatePreview(temp)
end

local function onEnter(o)
	GameTooltip:SetOwner(o, "ANCHOR_RIGHT");

	-- Internationalize !!!
	GameTooltip:SetText(o._tooltip_text) 

	GameTooltip:Show()
end

local function onLeave(o)
	GameTooltip:Hide()
end

local function onValueChanged(o)
	updateSliderText(o)
	local temp = _G[o._addon_name]:CreateOrGetTemp()
	temp._tooltip_order = o:GetValue()
end

local function onPullDateClick(cb)
	local temp = _G[cb._addon_name]:CreateOrGetTemp()
	temp._pullDate.checked = cb:GetChecked()
end

local function onDiscardClick(cb)
	local temp = _G[cb._addon_name]:CreateOrGetTemp()
	temp._discard.checked = cb:GetChecked()
end

local function doNothing(button)
end

local function verifySV(sv,addon)
	return sv -- Simple check for now.
end

local function onEvent(self,event,arg)
	if (event == "ADDON_LOADED") then
		local lc_name = string.lower(arg)
		if ( lc_name == string.lower(self._addon_name) ) then
			self:UnregisterEvent("ADDON_LOADED")

			if IsAddOnLoaded("EnhTooltip") then
				DEFAULT_CHAT_FRAME:AddMessage(
						"|c"..lib.nameColor..self._addon_name..": |rDetected the deprecated |c"..lib.evilColor.."EnhTooltip|r AddOn. |c"..lib.saintColor.."GoingPrice|r now uses |c"..lib.saintColor.."LibExtraTip|r. Please disable |c"..lib.evilColor.."EnhTooltip|r or use an earlier version of the |c"..lib.saintColor.."GoingPrice|r base AddOn.")
			end

			local addon = _G[self._addon_name]
			local sv = nil

			-- Verify the integrity of SV.
			-- If it fails verification then reset it to default values.
			--
			sv = _G[self._addon_name.."_SV"]

			if not verifySV(sv,addon) then
				_G[self._addon_name.."_SV"] = addon:CopyState(addon)
				sv = _G[self._addon_name.."_SV"]
			else	
				-- Make sure existing saved variable (SV) is up-to-date

				-- Look for any new stats
				--
				for _,k in ipairs(addon._keys) do
					if not string.match(k, "^_.*") and not sv[k] then
						sv[k] = {}
						sv[k].checked = false
						sv[k].keep = false
					end
				end
---[[
				-- Now remove any deprecated SV stats
				--
				for k,_ in pairs(sv) do
					if not string.match(k, "^_.*") and not addon[k] then
						removeLinkedListKey(sv,k)
						sv[k] = nil
					end
				end
--]]
				-- Now update SV's reverse map.
				--
				sv._index = nil
				sv._index = {}

				for k,i in pairs(addon._index) do
					sv._index[k] = i
				end

				if not sv._tooltip_order then
					sv._tooltip_order = addon._tooltip_order
				end

				for k,v in pairs(sv._reKeep) do
					if k == "_pullDate" then
						sv._pullDate.keep = true
					else
						sv[k].keep = true
					end
				end
		
				sv._reKeep = {}

				-- At this point SV is up to date. Now see if we have discarded
				-- statistics. If so, re-index and shrink.
				--
				for k,_ in pairs(sv._index) do
					if not sv[k].keep then
						local newSize,newToOldMap,newIndex
							= addon:ReIndex()
						addon:ShrinkData(newSize,newToOldMap,newIndex)
						break
					end
				end
			end

			addon:LoadPanelUI(sv)

			lib_tt:AddCallback(addon:GetCallback(),sv._tooltip_order)
			DEFAULT_CHAT_FRAME:AddMessage("|c"..lib.nameColor ..
						self._addon_name.." loaded")
		end
	end
end

function lib:CreateDataSource(source,url)

	local baseAddOnName = "GoingPrice"
	local addonName = baseAddOnName.."_"..source
	local addon = _G[addonName] -- addon's table object should have been already defined.

	addon._name = addonName
	addon._source = source

	setmetatable(addon, self)
	self.__index = self

	-- Create the addon's frame
	local frame = CreateFrame("Frame",addonName.."_Frame")
	addon._frame = frame -- We should keep this around.
	frame._addon_name = addonName -- We'll need the addon's name for 'onEvent'.
	frame:RegisterEvent("ADDON_LOADED")
	frame:SetScript("OnEvent", onEvent)

	-- Now create the addon's sub-panel.
	-- Let it be REFERENCED by a global variable named <addon name>_Panel
	-- BUT we will set its 'name' member to <addon name> in case we need
	-- to get the addon object from the panel object by using _G[panel.name]
	--
	local panel = CreateFrame("Frame",addon._name.."_Panel")
	panel.name = addon._name
	panel.parent = baseAddOnName

	local fs_left = panel:CreateFontString()

	fs_left:SetPoint('TOPLEFT', panel, 'TOPLEFT', 20, -20)
	fs_left:SetFontObject(GameFontNormalLarge)
	fs_left:SetText(baseAddOnName.." - "..source)

    for i,k in ipairs(addon._keys) do

        local cb = CreateFrame(	"CheckButton",
								addonName.."_StatCb_"..i,
								panel,
								"UICheckButtonTemplate")
		cb._addon_name = addonName

        getglobal(cb:GetName() .. "Text"):SetText(k) -- USE .text.<lang> instead of k
		cb._key = k
		cb._tooltip_text = "Check/Uncheck to Show/Hide this statistic"

        cb:SetPoint("TOPLEFT", panel, "TOPLEFT", 20, -20 - (25 * i))
		cb:SetScript("OnClick",onStatisticClick)
		cb:SetScript("OnEnter",onEnter)
		cb:SetScript("OnLeave",onLeave)

		-- Create (button) frames stats
		--
		local button = CreateFrame("Button",
					addonName.."_PreviewItem_"..i,
					panel,
					"OptionsListButtonTemplate") 
					--"UIPanelButtonTemplate") 

		button._addon_name = addonName

		-- We will call SetPoint in the update function.

        button._tooltip_text = "Preview of statistics\nand the order they\nwill appear in"

		button:SetScript("OnClick",doNothing)
		button:SetScript("OnEnter",onEnter)
		button:SetScript("OnLeave",onLeave)

		--button:SetNormalTexture([[Interface\BUTTONS\UI-Listbox-Highlight2]])
		--button:SetNormalFontObject("GameFontHighlight")
        button:SetText(k)
		button:SetWidth(150)
		button:SetHeight(16)
    end

	local slider = CreateFrame(	"Slider",
								addonName.."_Slider",
								panel,
								"OptionsSliderTemplate")
	slider._addon_name = addonName

	slider:SetMinMaxValues(0,1000)
	slider:SetValueStep(10)
	slider._basetext = "|cffffd100Placement " 
	_G[slider:GetName().."Text"]:SetText(slider._basetext)
	_G[slider:GetName().."Low"]:SetText("Higher")
	_G[slider:GetName().."High"]:SetText("Lower")

	slider:SetPoint("BOTTOM", panel,"BOTTOM",0,50)

	slider._tooltip_text = "Move |cffff0000"..source.."|r statistics Higher/Lower\nin the tooltip window."

	slider:SetScript("OnEnter",onEnter)
	slider:SetScript("OnLeave",onLeave)
	slider:SetScript("OnValueChanged",onValueChanged)

	slider:Show()

	local pullDate = CreateFrame(	"CheckButton",
									addonName.."_PullDateCb",
									panel,
									"UICheckButtonTemplate")
	pullDate._addon_name = addonName

	getglobal(pullDate:GetName() .. "Text"):SetText("Show Pull Date") -- Internationalize
    pullDate:SetPoint("BOTTOMLEFT", panel, "BOTTOMLEFT", 25, 5)

	pullDate._tooltip_text = "Show the date that the above\nstatistics were mined from\n|cffff0000 "..url.."|r"

	pullDate:SetScript("OnClick",onPullDateClick)
	pullDate:SetScript("OnEnter",onEnter)
	pullDate:SetScript("OnLeave",onLeave)

	pullDate:Show()

	local discard = CreateFrame(	"CheckButton",
									addonName.."_DiscardCb",
									panel,
									"UICheckButtonTemplate")
	discard._addon_name = addonName

	getglobal(discard:GetName() .. "Text"):SetText("Discard Unchecked") -- Internationalize
    discard:SetPoint("BOTTOMRIGHT", panel, "BOTTOMRIGHT", -125, 5)

	discard._tooltip_text = "Check here to clear unchecked statistics from addon memory.\n\n|c"..lib.nameColor.."Note:|r Discarding data might take a few seconds. |c"..lib.saintColor.."\n\nBut:|r It will also shrink "..addon._name.."'s memory usage.\n\nRe-check them and type |c"..lib.nameColor.."\"/console reloadui\"|r to get them back."

	discard:SetScript("OnClick",onDiscardClick)
	discard:SetScript("OnEnter",onEnter)
	discard:SetScript("OnLeave",onLeave)

	discard:Show()

	panel.okay = okay
	panel.cancel = cancel
	panel.default = default

    InterfaceOptions_AddCategory(panel)
end

function lib:ChangeTooltipOrder(order)
    --DEFAULT_CHAT_FRAME:AddMessage("|c"..lib.nameColor.."About to change order to "..order)

	local callback = self:GetCallback()

    -- Take a peek at the first element of sortedCallback to check its type
    if type(lib_tt.sortedCallbacks[1]) == "function" then
        -- Do it the easy way
        lib_tt:RemoveCallback(callback)
        lib_tt:AddCallback(callback,order)
        --DEFAULT_CHAT_FRAME:AddMessage("|c"..lib.nameColor.."Done the EASY way!")
        return true
    elseif type(lib_tt.sortedCallbacks[1]) == "table" then
        -- Do it the hard way
        -- Linearly search for my callback's table-counter-part key
        for i,c in ipairs(lib_tt.sortedCallbacks) do
            if c.callback == callback then
                lib_tt.callbacks[c] = order
                local callbacks = lib_tt.callbacks
                sortFunc = function(a,b)
                    return callbacks[a] < callbacks[b]
                end
                table.sort(lib_tt.sortedCallbacks,sortFunc)
                --DEFAULT_CHAT_FRAME:AddMessage("|c"..lib.nameColor.."Done the HARD way!")
                return true
            end
        end
    else
        return nil
    end
end
